#version 410 core

//in vec4 viewSpace;
in vec3 Position_worldspace;

out vec3 color;

const float PI	 	= 3.1415;
const float EPSILON	= 1e-3;

// sea
uniform int ITER_GEOMETRY = 2;
uniform int ITER_FRAGMENT = 5;
uniform float SEA_HEIGHT = 0.8;
uniform float SEA_CHOPPY = 4.0;
uniform float SEA_SPEED = 1.8;
uniform float SEA_FREQ = 0.16;
const vec3 SEA_BASE = vec3(0.1,0.19,0.22);
const vec3 SEA_WATER_COLOR = vec3(0.8,0.9,0.6);

mat2 octave_m = mat2(1.6,1.2,-1.2,1.6);

uniform vec3 LightPosition_worldspace;  
uniform vec3 CameraPosition;     
uniform float Time;
      
float SEA_TIME = Time * SEA_SPEED;    

// FOG

uniform vec3 FogColor = vec3(0.45, 0.59, 0.68);
const float FogDensity = 0.05;

// END FOG                    

// Reflection
uniform sampler2D ReflectionTextureSampler;                            
uniform float Shipheight;
    

mat3 fromEuler(vec3 ang) {
	vec2 a1 = vec2(sin(ang.x),cos(ang.x));
    vec2 a2 = vec2(sin(ang.y),cos(ang.y));
    vec2 a3 = vec2(sin(ang.z),cos(ang.z));
    mat3 m;
    m[0] = vec3(a1.y*a3.y+a1.x*a2.x*a3.x,a1.y*a2.x*a3.x+a3.y*a1.x,-a2.y*a3.x);
	m[1] = vec3(-a2.y*a1.x,a1.y*a2.y,a2.x);
	m[2] = vec3(a3.y*a1.x*a2.x+a1.y*a3.x,a1.x*a3.x-a1.y*a3.y*a2.x,a2.y*a3.y);
	return m;
}

float hash( vec2 p ) {
    float h = dot(p,vec2(127.1,311.7));	
    return fract(sin(h));//*43758.5453123);
}

float noise( in vec2 p ) {
    vec2 i = floor( p );
    vec2 f = fract( p );	

    vec2 u = f*f*(3.0-2.0*f);

    return -1.0+2.0*mix( 
                mix( hash( i + vec2(0.0,0.0) ), 
                     hash( i + vec2(1.0,0.0) ), 
                        u.x),
                mix( hash( i + vec2(0.0,1.0) ), 
                     hash( i + vec2(1.0,1.0) ), 
                        u.x), 
                u.y);
}



float sea_octave(vec2 uv, float choppy) {
    uv += noise(uv);

    vec2 wv = 1.0-abs(sin(uv)); 

    vec2 swv = abs(cos(uv));  

    wv = mix(wv,swv,wv);

    return pow(1.0-pow(wv.x * wv.y,0.65),choppy);
}

// like map + higher resolution (normal generation)
float map_detailed(vec3 p) {
    float freq = SEA_FREQ;
    float amp = SEA_HEIGHT;
    float choppy = SEA_CHOPPY;
    vec2 uv = p.xz; uv.x *= 0.75;
   
    float d, h = 0.0;    
    for(int i = 0; i < ITER_FRAGMENT; i++) {
   
    	d = sea_octave((uv+SEA_TIME)*freq,choppy);
   
    	d += sea_octave((uv-SEA_TIME)*freq,choppy);
        
        h += d * amp; 
        
    	uv *= octave_m;
        
        freq *= 1.9; 
        amp *= 0.22; 
		
        choppy = mix(choppy,1.0,0.2);
    }
    return p.y - h;
}

vec3 getNormal(vec3 p, float eps) {
    vec3 n;
    n.y = map_detailed(p); 
    n.x = map_detailed(vec3(p.x+eps,p.y,p.z)) - n.y; 
    n.z = map_detailed(vec3(p.x,p.y,p.z+eps)) - n.y; 
    n.y = eps; 
    return normalize(n);
}



vec3 getSkyColor(vec3 e) {
    e.y = max(e.y,0.0);
    vec3 ret;
    ret.x = pow(1.0-e.y,2.0);
    ret.y = 1.0-e.y;
    ret.z = 0.6+(1.0-e.y)*0.4;
    return ret;
}

float diffuse(vec3 n,vec3 l,float p) {
    return pow(dot(n,l) * 0.4 + 0.6,p);
}

float specular(vec3 n,vec3 l,vec3 e,float s) {    
    float nrm = (s + 8.0) / (3.1415 * 8.0);
    return pow(max(dot(reflect(e,n),l),0.0),s) * nrm;
}

// p: point on ocean surface to get color for
// n: normal on ocean surface at <p>
// l: light (sun) direction
// eye: ray direction from camera position for this pixel
// dist: distance from camera to point <p> on ocean surface
vec3 getSeaColor(vec3 p, vec3 n, vec3 l, vec3 eye, vec3 dist) {  
    float fresnel = 1.0 - max(dot(n,-eye),0.0);
    fresnel = pow(fresnel,3.0) * 0.65;
        
    vec3 reflected = getSkyColor(reflect(eye,n));    
    
    vec3 refracted = SEA_BASE + diffuse(n,l,80.0) * SEA_WATER_COLOR * 0.12; 
    
    vec3 color = mix(refracted,reflected,fresnel);
    
    float atten = max(1.0 - dot(dist,dist) * 0.001, 0.0);
    color += SEA_WATER_COLOR * (p.y - SEA_HEIGHT) * 0.18 * atten;
    
    color += vec3(specular(n,l,eye,60.0));
    
    return color;
}

void main(void) { 

	// FOG
	vec3 dist = Position_worldspace - CameraPosition;
	float distf = dot(dist,dist)*0.01;
	float fogFactor = 1.0 /exp( (distf * FogDensity)* (distf * FogDensity));
	fogFactor = clamp( fogFactor, 0.0, 1.0 );
	if (fogFactor < 0.001) {
		// Don't care about the rest, won't see anyway...
		color = FogColor;
		return;
	}

	vec3 lightDir = normalize(LightPosition_worldspace- Position_worldspace);
	vec3 normal = getNormal(Position_worldspace, dot(dist,dist)*0.0001);
	color = getSeaColor(Position_worldspace, normal, lightDir, normalize(dist), dist);
	
	// FOG
	
	//Reflection
	vec3 eyeDir = normalize(CameraPosition - Position_worldspace);
	vec3 reflectionVec = reflect(eyeDir, normal);
	
	//float height = map_detailed(vec3(0,0,0));
	
	//Get point on plane:
	float d = (Position_worldspace.y*Shipheight)/reflectionVec.y;
	vec3 intersect = d*reflectionVec+Position_worldspace;
	
	
	
	vec3 color2 = texture2D( ReflectionTextureSampler, clamp(intersect.zx / -10 +0.5,vec2(0,0) , vec2(1,1))).rgb;
	//color = color2;
	//return;
	//float dist2 = distance(Position_worldspace, vec3(0,0,0))/4;
	//dist2 = clamp(dist2,0,1);
	//color2 = mix(color2, vec3(0,0,0), dist2);
	//color = vec3(dist2,dist2,dist2);
	
	color = (2*color+color2)/3;
	
	
	//Fog mixing
	color = mix(FogColor, color, fogFactor);
	
} 